﻿Imports ThinkGeo.UI.WinForms
Imports ThinkGeo.Core
Imports System.Threading
Imports ThinkGeo.Core.Async
Public Class Form1
  Public WithEvents MainMap As CustomMapView

  Public Sub New()

    ' This call is required by the designer.
    InitializeComponent()

  End Sub

  Private Async Sub Form1_Load(sender As Object, e As EventArgs) Handles Me.Load
    MainMap = New CustomMapView With {
      .Dock = DockStyle.Fill
    }
    Controls.Add(MainMap)

    ZoomLevelHelper.InitZoomLevels()

    ZoomLevelHelper.ApplyCustomZoomLevels(MainMap.ZoomLevelSet, ZoomLevelHelper.EnumSymbolType.None)

    MainMap.MapUnit = GeographyUnit.Meter
    MainMap.ExtentOverlay.PanMode = MapPanMode.StandardPanning

    mAddShapefile()

    Await MainMap.RefreshAsync

    mTestSerialization()

  End Sub
  Private Async Function mAddWmtsLayer() As Task
    Dim oWmtsLayer As New WmtsAsyncLayer(New Uri("https://basemap.nationalmap.gov/arcgis/rest/services/USGSTopo/MapServer/WMTS?"))
    With oWmtsLayer
      .DrawingExceptionMode = DrawingExceptionMode.DrawException
      .CapabilitesCacheTimeout = New TimeSpan(0, 0, 0, 1)
      .ActiveLayerName = "USGSTopo"
      .ActiveStyleName = "default"
      .OutputFormat = "image/png"
      .TileMatrixSetName = "GoogleMapsCompatible"
      Dim oOverlay As New LayerOverlay With {.Name = "Layers"}
      oOverlay.Layers.Add(oWmtsLayer)

      'mAddShapefile(oOverlay)

      MainMap.Overlays.Add("Layers", oOverlay)
      MainMap.CurrentExtent = New RectangleShape(-20037507.8427882, 20037508.2314698, 20037507.8407383, -20037508.2527101)
      Await MainMap.RefreshAsync(MainMap.InteractiveOverlays)
      Await MainMap.RefreshAsync
    End With
  End Function
  Private Sub mAddShapefile()
    Dim oShapefile As New ShapeFileFeatureLayer("..\..\Data\AreasOfConcern.shp")
    Dim oAreaStyle1 As AreaStyle = AreaStyle.CreateSimpleAreaStyle(GeoColors.Transparent, GeoColors.Blue, 1)
    Dim oAreaStyle2 As AreaStyle = AreaStyle.CreateSimpleAreaStyle(GeoColors.Transparent, GeoColors.Red, 1)
    ZoomLevelHelper.ApplyCustomZoomLevels(oShapefile.ZoomLevelSet, ZoomLevelHelper.EnumSymbolType.AreaSymbol)

    Dim iFirstHalf As Integer = 20
    Dim iSecondHalf = 38
    Dim iCounter As Integer = 0

    For Each oZoomLevel As ZoomLevel In oShapefile.ZoomLevelSet.CustomZoomLevels
      If iCounter < 20 Then
        oZoomLevel.DefaultAreaStyle = oAreaStyle1
      Else
        oZoomLevel.DefaultAreaStyle = oAreaStyle2
      End If
      iCounter = iCounter + 1
    Next oZoomLevel
    oShapefile.DrawingQuality = DrawingQuality.HighSpeed
    Dim oProj As New ProjectionConverter(102003, 3857)
    oShapefile.FeatureSource.ProjectionConverter = oProj

    Dim oOverlay As New LayerOverlay() With {.Name = "Layers"}
    MainMap.Overlays.Add("Layers", oOverlay)
    'Dim oOverlay = CType(MainMap.Overlays("Layers"), LayerOverlay)
    oOverlay.Layers.Add(oShapefile)
    MainMap.CurrentExtent = New RectangleShape(-9762065.08512668, 3744836.02203581, -9681550.03146195, 3697034.6992771)

  End Sub
  Private Sub MainMap_CurrentExtentChanged(sender As Object, e As CurrentExtentChangedMapViewEventArgs) Handles MainMap.CurrentExtentChanged
    Dim o = MainMap.CurrentExtent
  End Sub

  Private Sub mTestSerialization()
    Dim oOverlay As LayerOverlay = CType(MainMap.Overlays("Layers"), LayerOverlay)
    Dim oShapefile = CType(oOverlay.Layers(0), ShapeFileFeatureLayer)
    Dim sSerializationPath As String = "..\..\Data\SerializationTest.xml"
    Dim oGeoSerializer As New GeoSerializer

    '***** Set the 'Name' property on the shapefile
    oShapefile.Name = "AreasOfConcern"
    oShapefile.Close()
    '***** Serialize the shapefile
    oGeoSerializer.Serialize(oShapefile, sSerializationPath)

    '***** Deserialize the shapefile
    Dim oObjLayer = oGeoSerializer.Deserialize(sSerializationPath, IO.FileAccess.Read)

    '***** Note that the 'Name' public property and back field name are not populated 
    '***** in the deserialized shapefile
    Dim oDeserializedShapefile = CType(oObjLayer, ShapeFileFeatureLayer)


  End Sub

End Class
